local Addon, private = ...

setfenv(1, private)

-- Private methods
-- ============================================================================

local bookmarks = {
	-- [*] = { module, path, title }
}

local linksCache = {
	-- [frame] = true
}

local frame
local links = { }

local function createLink(parent)
	local link = next(linksCache)
	if(link) then
		linksCache[link] = nil
		link:SetVisible(true)
		return link
	else
		local link = UI.CreateFrame("Text", "", parent)
		return link
	end
end

local function rebuildFrames(self)
	for i = 1, #bookmarks do
		local link = links[i]
		local bookmark = bookmarks[i]
		if(not link) then
			link = createLink(self)
			if(i == 1) then
				link:SetPoint("TOPLEFT", self, "TOPLEFT", Const.padding, 10)
			else
				link:SetPoint("TOPLEFT", links[i - 1], "BOTTOMLEFT")
			end
			links[i] = link
		end
		link:SetText(string.format(Const.moduleLinkFormat, bookmark[1], bookmark[2], bookmark[3]), true)
	end
	for i = #bookmarks + 1, #links do
		local link = links[i]
		links[i] = nil
		link:SetVisible(false)
		linksCache[link] = true
	end
end

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, function(handle, identifier)
	if(identifier == Addon.identifier) then
		bookmarks = _G.ApiBrowserBookmarks or { }
	end
end, "")

Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, function(handle, identifier)
	if(identifier == Addon.identifier) then
		_G.ApiBrowserBookmarks = bookmarks
	end
end, "")

-- Public methods
-- ============================================================================

local function Add(self, module, path, title)
	if(self:IsBookmarked(module, path)) then
		return
	end
	
	bookmarks[#bookmarks + 1] = { module, path, title }
	table.sort(bookmarks, function(a, b) return a[3] < b[3] end)
	rebuildFrames(self)
end

local function IsBookmarked(self, module, path)
	for i = 1, #bookmarks do
		if(bookmarks[i][1] == module and bookmarks[i][2] == path) then
			return true
		end
	end
	return false
end

local function Remove(self, module, path)
	for i = 1, #bookmarks do
		if(bookmarks[i][1] == module and bookmarks[i][2] == path) then
			table.remove(bookmarks, i)
			rebuildFrames(self)
			return
		end
	end
end

local frame = function(parent)
	local self = UI.CreateFrame("Mask", "", parent)
	self:SetBackgroundColor(0, 0, 0, 0.5)
	frame = self

	self.Add = Add
	self.IsBookmarked = IsBookmarked
	self.Remove = Remove
	
	rebuildFrames(self)
	return self
end

function BookmarksFrame(parent)
	if(type(frame) == "function") then
		return frame(parent)
	else
		return frame
	end
end
