local Addon, private = ...

setfenv(1, private)
Catalog = { }

local catalogs = { }

-- Private methods
-- ============================================================================

local strmatch = string.match
local split = string.split
local strfind = string.find
local function categorize(name)
	local object = strmatch(name, "^([^:]+):.+")
	if(object) then
		local t = split(object, ".")
		if(#t >= 2) then
			return t[1], "method"
		else
			return object, "method"
		end
	else
		local t = split(name, ".")
		if(#t >= 3) then
			return t[2], "normal"
		elseif(#t >= 2) then
			return t[1], "normal"
		else
			return name, "global"
		end
	end
end

-- Public methods
-- ============================================================================

function Catalog.GetCatalog(name)
	return catalogs[name] or { }
end

function Catalog.SetCatalog(name, catalog)
	catalogs[name] = catalog
end

local function map(i, name)
	if(i == 1) then
		return "category:" .. name
	elseif(i == 2) then
		return "object:" .. name
	else
		return name
	end
end

function Catalog.BuildCatalogFromRiftLikeFormat(name, index)
	if(catalogs[name]) then
		return
	end
	local catalog = {
		{}, -- First section are symbols by global category (e.g. Ability, Item, Unit, ...)
		{}, -- Second sections are symbols with instance methods (e.g. Frame, Context, ...)
		{}, -- Third category are unbound globals
		map = { -- Used for mapping paths to other paths where necessary
			-- [name] = name
		},
	}

	for name in pairs(index) do
		local category, type = categorize(name)
		if(type == "method") then
			local index = map(2, category)
			catalog[2][index] = catalog[2][index] or { }
			catalog[2][index][name] = true
			catalog.map[index] = category
		elseif(type == "global") then
			catalog[3][category] = true
		else
			local index = map(1, category)
			catalog[1][index] = catalog[1][index] or { }
			catalog[1][index][name] = true
			catalog.map[index] = category
		end
	end
	-- If there is only a single category make the entries all global
	local index, entries = next(catalog[1])
	if(entries and not (next(catalog[1], index))) then
		for name in pairs(entries) do
			catalog[3][name] = true
		end
		catalog.map[index] = nil
		catalog[1][index] = nil
	end
	-- Globals matching catagory or object names belong there
	for global in pairs(catalog[3]) do
		local index1 = map(1, global)
		local index2 = map(2, global)
		local c = catalog[1][index1] or catalog[2][index2]
		if(c) then
			catalog[3][global] = nil
			c[global] = true
		end
	end
	catalogs[name] = catalog
	return catalog
end
