local Addon, private = ...

setfenv(1, private)

-- Private methods
-- ============================================================================

local function typeFormatCallback(module, globals, objects, type)
	if(globals[type]) then
		return LibWiki.MakeHyperlinkMarkup(module, type, type)
	else
		local index = "object:" .. type
		if(objects[index]) then
			return LibWiki.MakeHyperlinkMarkup(module, index, type)
		end
	end
	return type
end

-- Public methods
-- ============================================================================

local function CreatePage(self, parent, path)
	-- Just in case some pages take longer to create
	pcall(Command.System.Watchdog.Quiet)
	
	local mapped = (self.catalog.map or { })[path] or path
	local title = mapped
	for i = 1, #self.catalog do
		local entry = self.catalog[i][path]
		if(type(entry) == "table") then
			return CategoryPage(parent, self.pageBuilder, self.module, mapped, entry, self.inspectCallback), title
		end
	end
	
	local frame
	local doc = self.inspectCallback(path)
	if(doc) then
		frame = DetailPage(parent, self.pageBuilder, self.module, path, doc, function(...)
			return typeFormatCallback(self.module, self.catalog[3], self.catalog[2], ...)
		end)
	else
		frame, title = self.backupCallback(parent, self.pageBuilder, path)
		if(not frame) then
			frame = DetailPage(parent, self.pageBuilder, self.module, path, "Page not found.")
			title = mapped
		end
	end
	return frame, title
end

local function RecyclePage(self, page)
	-- TODO: this won't always work
	self.pageBuilder:RecyclePage(page, MainContext())
end

function RiftLikeCatalogPageFactory(catalog, module, style, inspectCallback, backupCallback)
	assert(type(inspectCallback) == "function")
	assert(type(backupCallback) == "function")
	return {
		module = module,
		catalog = catalog,
		inspectCallback = inspectCallback,
		backupCallback = backupCallback,
		pageBuilder = _G[Addon.identifier].DefaultPageBuilder,
		
		CreatePage = CreatePage,
		RecyclePage = RecyclePage,
	}
end
