local Addon, private = ...

setfenv(1, private)

local function indexForMembers(module, members, docs)
	local enabledMethodFmt = Const.methodIndexFormat
	local disabledMethodFmt = Const.disabledMethodIndexFormat
	local disabledFmt = Const.disabledFormat
	for i = 1, #members do
		local name = members[i]
		local frame, method = string.match(name, "^([^:]+):(.*)")
		local doc = docs[name]
		local disabled = doc and doc.disabled
		if(frame and method) then
			name = string.format(disabled and disabledMethodFmt or enabledMethodFmt, frame, method)
		elseif(disabled) then
			name = string.format(disabledFmt, name)
		end
		members[i] = LibWiki.MakeHyperlinkMarkup(module, members[i], name)
	end
	return members
end

local function separateCategoryMembersByType(members, inspector)
	local types = { }
	local docs = { }
	for member in pairs(members) do
		local doc = inspector(member)
		if(doc and type(doc) ~= "table") then
			doc = { }
		end
		if(doc) then
			local type = ((doc.order or doc.signature) and "event") or doc.type or "misc"
			local t = types[type] or {}
			t[#t + 1] = member
			types[type] = t
			docs[member] = doc
		end
	end
	for type, members in pairs(types) do
		table.sort(members)
	end
	return types, docs
end

function CategoryPage(parent, pageBuilder, module, title, members, inspector)
	local self = pageBuilder:BeginPage(parent, module)
	
	pageBuilder:BuildHeader1(self, title)
	local types, docs = separateCategoryMembersByType(members, inspector)
	
	if(types.type) then
		pageBuilder:BuildHeader2(self, "Types")
		pageBuilder:BuildIndex(self, indexForMembers(module, types.type, docs), true)
	end
	if(types["function"]) then
		pageBuilder:BuildHeader2(self, "Functions")
		pageBuilder:BuildIndex(self, indexForMembers(module, types["function"], docs), true)
	end
	if(types.event) then
		pageBuilder:BuildHeader2(self, "Events")
		pageBuilder:BuildIndex(self, indexForMembers(module, types.event, docs), true)
	end
	if(types.misc) then
		pageBuilder:BuildHeader2(self, "Other")
		pageBuilder:BuildIndex(self, indexForMembers(module, types.misc, docs), true)
	end
	
	pageBuilder:EndPage(self)
	return self
end
