local Addon, private = ...

setfenv(1, private)

-- Private methods
-- ============================================================================

local processesedKeys = {
	readable = true,
	summary = true,
	signatures = true,
	signature = true,
	order = true,
	members = true,
	name = true,
	type = true,
	removed = true,
	parameter = true,
	result = true,
--	noSecureFrameAndEnvironment = true,
	throttleGlobal = true,
	interaction = true,
}

local paragraphs = {
	interaction = "Required Interaction Flag",
}

-- Public methods
-- ============================================================================

function DetailPage(parent, pageBuilder, module, title, data, typeFormatCallback)
	local self = pageBuilder:BeginPage(parent, module)
	
	local padding = Const.padding
	
	title = data.throttleGlobal and string.format(Const.titleAdditionFormat, title) or title
--	title = data.noSecureFrameAndEnvironment and string.format(Const.secureTitleFormat, title) or title
	title = data.removed and string.format(Const.deprecatedTitleFormat, title) or title
	pageBuilder:BuildHeader1(self, title, true)
	if(type(data) == "string") then
		pageBuilder:BuildParagraph(self, data)
	elseif(type(data) == "table") then
		if(data.summary) then
			pageBuilder:BuildParagraph(self, data.summary, true)
		end
		if(data.signatures) then
			pageBuilder:BuildHeader2(self, "Signatures")
			pageBuilder:BuildFunctionSignatures(self, data.signatures, data.parameter or {}, data.result or {}, typeFormatCallback)
		end
		if(data.signature) then
			pageBuilder:BuildHeader2(self, "Signature")
			if(type(data.signature == "string")) then
				pageBuilder:BuildFunctionSignatures(self, { data.signature }, data.parameter or {}, {}, typeFormatCallback)
			else
				pageBuilder:BuildFunctionSignatures(self, data.signature, data.parameter or {}, {}, typeFormatCallback)
			end
		end
		if(data.members) then
			pageBuilder:BuildHeader2(self, "Members")
			pageBuilder:BuildKeyValueTable(self, data.members, true)
		end
		for k, v in pairs(data) do
			if(paragraphs[k]) then
				pageBuilder:BuildHeader2(self, paragraphs[k])
				pageBuilder:BuildParagraph(self, v, true)
			end
		end
		for k, v in pairs(data) do
			if(not processesedKeys[k]) then
				pageBuilder:BuildHeader2(self, paragraphs[k] or k)
				if(type(v) == "table") then
					pageBuilder:BuildKeyValueTable(self, v, true)
				else
					pageBuilder:BuildParagraph(self, v, true)
				end
			end
		end
	end
	
	pageBuilder:EndPage(self)
	return self
end