local Addon, private = ...

setfenv(1, private)

-- Private methods
-- ============================================================================

local function indexForCatalogTable(map, members, module)
	local names = {}
	for name in pairs(members) do
		names[#names + 1] = name
	end
	table.sort(names)
	for i = 1, #names do
		local name = names[i]
		names[i] = LibWiki.MakeHyperlinkMarkup(module, name, map[name] or name)
	end
	return names
end

local function buildBody(page, module, title, pageBuilder, catalog, sectionNames)
	sectionNames = sectionNames or { }
	pageBuilder:BuildHeader1(page, title)
	if(catalog.summary) then
		pageBuilder:BuildParagraph(page, catalog.summary, true)
	end
	local map = catalog.map or { }
	local numCategories = 0
	for i = 1, #catalog do
		if(next(catalog[i])) then
			numCategories = numCategories + 1
		end
	end
	for i = 1, #catalog do
		if(next(catalog[i])) then
			if(numCategories > 1 and sectionNames[i]) then
				pageBuilder:BuildHeader2(page, sectionNames[i])
			end
			pageBuilder:BuildIndex(page, indexForCatalogTable(map, catalog[i], module), true)
		end
	end
end

local mainPage
local libsIndex
local mainPageBuilder

-- Public methods
-- ============================================================================

function RiftLikeCatalogIndexPage(parent, pageBuilder, catalog, module, sectionNames)
	local self = pageBuilder:BeginPage(parent, module)
	
	buildBody(self, module, module, pageBuilder, type(catalog) == "table" and catalog or Catalog.GetCatalog(catalog), sectionNames)
	pageBuilder:EndPage(self)
	return self
end
