local Addon, private = ...

setfenv(1, private)

-- Private methods
-- ============================================================================

local function defaultBackupCallback(module, parent, pageBuilder, path)
	if(path == Const.indexPath) then
		return RiftLikeCatalogIndexPage(parent, pageBuilder, module, module, {
			"By Category",
			"Objects",
			"Other",
		}), module
	else
		return nil
	end
end

-- Keep a list of all libraries and their main page path
local registeredLibraries = {
	-- [module] = path
}

-- Public methods
-- ============================================================================

Libraries = { }

function Libraries.Index()
	local names = { }
	for name, index in pairs(registeredLibraries) do
		names[#names + 1] = name
	end
	table.sort(names)
	for i = 1, #names do
		local name = names[i]
		names[i] = string.format(Const.moduleLinkFormat, name, registeredLibraries[name], name)
	end
	return names
end

function Libraries.AddLibraryWithRiftLikeCatalogIndex(module, name, summary, catalogIndex, inspectCallback, backupCallback)
	local catalog = Catalog.BuildCatalogFromRiftLikeFormat(module, catalogIndex)
	catalog.summary = summary
	LibWiki.RegisterPageFactory(module,
		RiftLikeCatalogPageFactory(catalog, module, nil,
			inspectCallback or function(path) return catalogIndex[path] end,
			backupCallback or function(...) return defaultBackupCallback(module, ...) end
		)
	)
	if(module ~= "Rift") then
		registeredLibraries[module] = Const.indexPath
		LibrariesChanged()
	end
end

function Libraries.AddLibraryWithFactory(module, name, factory)
	if(module ~= "Rift") then
		LibWiki.RegisterPageFactory(module, factory)
		registeredLibraries[module] = name
		LibrariesChanged()
	end
end