do

local symbols = { }

symbols["Inspector"] = {
	summary = [[
A function which takes a single string argument representing the documentation path and returns the data to be displayed on a detail page.]],
	signatures = {
		"content = Inspector(path) -- string/table/nil <- string"
	},
	parameter = {
		["path"] = "The path of the documentation object.",
	},
	result = {
		["content"] = [[
Either a string, which is displayed on the detail page (HTML enabled) or a table with the same format as returned from the 'computer-readable' overload of <<Rift:Inspect.Documentation>>. See the table below for supported members (all members are optional). If the result is nil then either no page is displayed, or (if used with <<:ApiBrowser.AddLibraryWithRiftLikeCatalogIndex>>) a "Page not found" page is displayed.]],
	},
	["Recognized Members"] = {
		summary = "string: Text to be displayed on top of the page (HTML enabled).",
		signatures = "table: Array of function signatures (see <<LibWiki:PageBuilder:BuildFunctionSignatures>>).",
		parameter = "table: Function/Event parameter descripion (see <<LibWiki:PageBuilder:BuildFunctionSignatures>>).",
		result = "table: Function results description (see <<LibWiki:PageBuilder:BuildFunctionSignatures>>).",
		type = [[string: "function" or "type". Used as a hint in category index pages if your documentation can be grouped into categories.]],
		signature = [[table: Same as "signatures" but indicates an event instead of a function. Used as a hint in category index pages if your documentation can be grouped into categories.]],
		removed = "boolean: true if this symbol is deprecated.",
		noSecureFrameAndEnvironment = "boolean",
		throttleGlobal = "boolean",
		interaction = "string: required API interaction flags.",
		["(other)"] = [[
string: Text displayed as a paragraph (HTML enabled).

table: Listed as a key-value table in the same style as the one you are currently looking at. Both keys and values are HTML enabled. Nesting of tables is not supported.

In both cases the key is used as section header text.]],
	},
}

symbols["ApiBrowser.AddLibraryWithFactory"] = {
	summary = [[
Add a new library to ApiBrowser using a custom page factory if your documentation is not suited for ApiBrowser's automatic page generation.

It is your own job to create content pages, however some of the other ApiBrowser methods may make this a bit easier.]],
	signatures = {
		"ApiBrowser.AddLibraryWithFactory(module, name, factory) -- string, string, PageFactory",
	},
	parameter = {
		["module"] = "The name of the library/module. This is used to separate pages with the same page path and to enlist it in the sidebar.",
		["name"] = "The displayable name of the module. This is currently not used but this may change in the future.",
		["factory"] = "A custom page factory bypassing Apibrowser's automatic page generation (see <<LibWiki:object:PageFactory|PageFactory>>).",
	},
	result = {
	},
	type = "function",
}

symbols["ApiBrowser.AddLibraryWithRiftLikeCatalogIndex"] = {
	summary = [[
Add a new library to ApiBrowser based on the same format and structure as the built-in Rift documentation.]],
	signatures = {
		"ApiBrowser.AddLibraryWithRiftLikeCatalogIndex(module, name, summary, catalogIndex, inspectCallback, backupCallback) -- string, string, string/nil, table, Inspector/nil, PageFactory/nil",
	},
	parameter = {
		["module"] = "The name of the library/module. This is used to separate pages with the same page path and to enlist it in the sidebar.",
		["name"] = "The displayable name of the module. This is currently not used but this may change in the future.",
		["summary"] = "An optional summary to be displayed on the module's index page (only relevant if backupCallback is nil or if later used with <<:ApiBrowser.CreateRiftLikeCatalogIndexPage>>).",
		["catalogIndex"] = "A table in the format {name = true} with all the pages for which the inspectCallback provides information. The table keys are categorized with the same algorithm as used for the Rift documentation catalog.",
		["inspectCallback"] = "A function taking a page path as string and returning displayable content (see <<:Inspector>> for details). If this value is nil it is replaced with a simple table lookup inside catalogIndex.",
		["backupCallback"] = [[In case inspectCallback returns nil the backup is called where you can handle page paths which are not function documentations but arbitrary pages with custom content. If this parameter is nil then ApiBrowser tries to provide a main index page as well as category index pages if your documentation can be grouped into categories and shows a "Page not found" message for any page paths it cannot resolve.]],
	},
	result = {
	},
	type = "function",
}

symbols["ApiBrowser.CreateCategoryPage"] = {
	summary = [[
Create a new category index page. These are used to give an overview of a category inside a larger documentation and are used extensively for Rift's documentation pages (e.g. <<Rift:category:Cursor|Cursor>>, <<Rift:category:Item|Item>>, <<Rift:category:Unit|Unit>>, ...).]],
	signatures = {
		"page = ApiBrowser.CreateCategoryPage(parent, pageBuilder, module, title, members, inspector) -- Frame <- Frame, PageBuilder, string, string, table, Inspector",
	},
	parameter = {
		["parent"] = "The frame acting as the parent of the page. Should be the content frame of a <<LibWiki:object:PageContainer|PageContainer>>.",
		["pageBuilder"] = "A PageBuilder object to be used for building the page (see <<LibWiki:object:PageBuilder|PageBuilder>>).",
		["module"] = "The name of the library/module. This is used to create hyperlinks to the correct detail pages.",
		["title"] = "The title for the page.",
		["members"] = "A table containing the members of the category to be shown in the format {name = true}.",
		["inspector"] = "An Inspector function used for grouping symbols by their type (e.g. functions, types, events, ...).",
	},
	result = {
		["page"] = "A frame to be displayed as a page inside a <<LibWiki:object:PageContainer|PageContainer>> instance.",
	},
	type = "function",
}

symbols["ApiBrowser.CreateDetailPage"] = {
	summary = [[
Create a new detail page. These are meant to show the detailed documentation of a function, type or other symbol (e.g. <<Rift:Inspect.Item.Detail>>, <<Rift:Frame:SetPoint>>, <<Rift:currencycategory>>, ...).]],
	signatures = {
		"page = ApiBrowser.CreateDetailPage(parent, pageBuilder, module, title, content, typeFormatCallback) -- Frame <- Frame, PageBuilder, string, table/string, function/nil",
	},
	parameter = {
		["module"] = "The name of the library/module containing this page. Used for automatic linking between pages.",
		["parent"] = "The frame acting as the parent of the page. Should be the content frame of a <<LibWiki:object:PageContainer|PageContainer>>.",
		["pageBuilder"] = "A PageBuilder object to be used for building the page (see <<LibWiki:object:PageBuilder|PageBuilder>>).",
		["title"] = "The title for the page.",
		["content"] = [[
The content to be displayed on the page.

string: A simple text content (HTML enabled).

table: A table in the format as described on the Inspector page. The data is fed into pageBuilder to create consistently looking detail pages.]],
		["typeFormatCallback"] = [[
An optional function taking a single string argument and returning a string (HTML enabled). This callback is invoked when ApiBrowser dissects the signature strings. Here you have the possibility to custom format these type strings. If it is nil the type strings are displayed unmodified.

If you are using <<:ApiBrowser.AddLibraryWithRiftLikeCatalogIndex>> then this is the place ApiBrowser tries to match the type names to relevant pages if it can find any.]],
	},
	result = {
		["page"] = "A frame to be displayed as a page inside a <<LibWiki:object:PageContainer|PageContainer>> instance.",
	},
	type = "function",
}

symbols["ApiBrowser.CreateRiftLikeCatalogIndexPage"] = {
	summary = [[
Create a new catalog index page for a Rift-like documentation catalog. An example for this is the start page of ApiBrowser listing the Rift documentation.]],
	signatures = {
		"page = ApiBrowser.CreateRiftLikeCatalogIndexPage(parent, pageBuilder, catalog, module, sectionNames) -- Frame <- Frame, PageBuilder, Catalog/string, string, table/nil",
	},
	parameter = {
		["parent"] = "The frame acting as the parent of the page. Should be the content frame of a <<LibWiki:object:PageContainer|PageContainer>>.",
		["pageBuilder"] = "A PageBuilder object to be used for building the page (see <<LibWiki:object:PageBuilder|PageBuilder>>).",
		["module"] = "The name of the library/module. This is used to create hyperlinks to the correct detail pages.",
		["catalog"] = "If a string it must refer to a catalog which was previously built using <<:ApiBrowser.BuildCatalogFromRiftLikeFormat>>, otherwise a table in the Catalog format.",
		["sectionNames"] = "An optional array of strings specifying the header texts used for the catalog sections.",
	},
	result = {
		["page"] = "A frame to be displayed as a page inside a <<LibWiki:object:PageContainer|PageContainer>> instance.",
	},
	type = "function",
}

symbols["ApiBrowser.DefaultPageBuilder"] = {
	summary = [[
Refers to the default PageBuilder instance used by the built-in ApiBrowser methods for generating pages. Use this in the implementation of your custom page factories to match the style of the other ApiBrowser pages.]],
}

symbols["ApiBrowser.PushPage"] = {
	summary = [[
Push a new page onto the ApiBrowser page stack and show its window if hidden.

This method is currently used to realize the modules sidebar inside ApiBrowser but is not restricted to this use and can be called from other addons to display certain pages in ApiBrowser.]],
	signatures = {
		"ApiBrowser.PushPage(module, path) -- string, string",
	},
	parameter = {
		["module"] = "The name of the library/module. This is used to create hyperlinks to the correct detail pages.",
		["path"] = "The path of the page to be displayed.",
	},
	result = {
	},
	type = "function",
}

symbols["ApiBrowser.BuildCatalogFromRiftLikeFormat"] = {
	summary = [[
Given a single table with all the API pages available create a catalog table compatible with <<:ApiBrowser.CreateRiftLikeCatalogIndexPage>> and other internal ApiBrowser mechanics.]],
	signatures = {
		"catalog = ApiBrowser.BuildCatalogFromRiftLikeFormat(module, index) -- Catalog <- string, table",
	},
	parameter = {
		["module"] = "The name of the library/module. This is used to create hyperlinks to the correct detail pages.",
		["index"] = "A table in the format {name = true} to generate the catalog from.",
	},
	result = {
		["catalog"] = "A Catalog table suitable to be used with other ApiBrowser methods and internal mechanics.",
	},
	type = "function",
}

symbols["Catalog"] = {
	summary = [[
ApiBrowser uses a concept called a Catalog to break down large documentation bases. A Catalog is a table with a certain hierarchical organziation of its contents and is automatically generated by <<:ApiBrowser.BuildCatalogFromRiftLikeFormat>>. However the format is simple enough to be created by custom means.

If you do not intend to use <<:ApiBrowser.BuildCatalogFromRiftLikeFormat>> or <<:ApiBrowser.CreateRiftLikeCatalogIndexPage>> then this information is probably irrelevant to you. <<:ApiBrowser.AddLibraryWithRiftLikeCatalogIndex>> handles all these details behind the scenes.

The structure of a catalog is an array of sections, each section being a table of symbols it contains.
The first three sections of a catalog have special meaning and are used for: "Categories", "Objects" and "Other".

"Categories" is at index 1 and is used to split up the large documentation base of Rift into categories like "Ability", "Item" or "Unit". It has the format {name = table}. Unless your page paths contain three parts (e.g. "Inspect.Item.Detail") this section is most likely empty. The values of this sections are tables in the format {name = true} containing the actual symbols which are part of this section.

"Objects" is at index 2 and populated from all index entries containing a colon indicating object methods (e.g. "Frame:SetVisible", "RiftWindow:GetContent", ...). It has the format {name = table}. If your documentation does not contain objects this section will be empty. The values of this section are tables in the format {name = true} containing the actual symbols which are part of this object.

"Other" is at index 3 and is filled with all the symbols which cannot be assigned to one of the other. Contrary to the other two sections the values are not tables but the entire section is of the format {name = true}.

If you are building your own catalog it is highly suggested you keep indices 1 - 3 for their intended purpose as ApiBrowser creates special hyperlinks for them to avoid name clashes (e.g. to distinguish between the category "Frame" and the "Frame" type). Sections which are left empty are not displayed by <<:ApiBrowser.CreateRiftLikeCatalogIndexPage>>. Any further sections are displayed without special hyperlink processing.

When ApiBrowser tries to match type names found in function signatures it searches sections 2 and 3 for relevant pages.

A catalog can have the optional string field "summary" displayed at the top of <<:ApiBrowser.CreateRiftLikeCatalogIndexPage>>.

An example catalog might look like the following (excerpt from Rift's API):
{
	summary = "Some text",
	{
		["Item"] = {
			["Command.Item.Split"]= true,
			["Event.Item.Slot"]= true,
			["Inspect.Item.Detail"]= true,
			["Inspect.Item.List"]= true,
			["Utility.Item.Slot.Guild"] = true,
			...
		},
		...
	},
	{
		["Frame"] = {
			["Frame"] = true,
			["Frame:SetParent"] = true,
			["Frame:GetWidth"] = true,
			["Frame:SetPoint"] = true,
			["Frame:ClearAll"] = true,
			...
		},
		...
	},
	{
		["Element"] = true,
		["dump"] = true,
		["ability"] = true,
		["faction"] = true,
		["handleEventFrame"] = true,
		...
	},
	...
}]],
}

symbols["Categorization"] = {
	summary = [[
Given a single table in the format {name = true} ApiBrowser uses the following algorithm to separate the entries into categories:

If the name contains a colon it is treated as an object method and grouped moved the respective object. In case the object name consists of multiple parts (e.g. "Frame.Event:XXX") the first part is used as object name.

Otherwise if the name has more than two components (e.g. "Inspect.Ability.Detail") the second component is used as category name. Otherwise the name has either two components or one. In the latter case the symbol is moved to the "Other" section unless it matches an object name, then it is moved to the object (e.g. the documentation for the "Frame" type is part of the "Frame" object).

If the symbol has two components it gets a bit tricky. If a category with the second component exists it is added there, otherwise the first component is the category. Unless there are multiple two-component symbols with the same second category, in which they are grouped there. This is quite some guesswork and tailored specifically for the Rift API, unless anyone comes up with a better solution.

If in the end there is only a single category its symbols are moved to the "Other" part to save an unnecessary indirection on the index page and many libraries have their functions embedded in some kind of global table with the library's name.]],
}

symbols["Hyperlinks"] = {
	summary = [[
Since ApiBrowser is powered by <<LibWiki:INDEX|LibWiki>> to create its pages you can use the LibWiki hyperlink markup almost everyhwere. See the LibWiki <<LibWiki:Hyperlinks>> page for details.

However there are some points you need to keep in mind when writing links for ApiBrowser:
<font color="#00ff00">
The automatically generated links for categories and objects as described in <<:Categorization>> are prefixed with "category:" and "object:" respectively to avoid name clashing with type names. Consider this in your links. For example use &lt;&lt;Rift:object:Frame|Frame&gt;&gt; instead of &lt;&lt;Rift:Frame&gt;&gt; since the former points to the <<Rift:object:Frame|member listing>> but the latter to the <<Rift:Frame|type>>.

The path of automatically created module index pages is "INDEX".</font>
]],
}

local summary = [[
ApiBrowser acts as a central area for gathering Rift related API documentation. It is however not limited to the in-game Rift API and supports extension by other third party libraries or tools for which documentation can be provided by their authors.

In order for your library to support this system make ApiBrowser an optional dependency of your library and then invoke the functions mentioned here to add your library documentation to ApiBrowser.

Tipp: Use (one or more) separate files containing your documentation and check for the existence of the global "ApiBrowser" object at the beginning of the file. If it doesn't exist "return" from the file so normal end users are spared the unnecesary loading time and memory consumpation. You can check ApiBrowser/doc/Example.lua for an idea of how such a typical documentation source file might look like.]]

local Addon = ...
_G[Addon.identifier].AddLibraryWithRiftLikeCatalogIndex(Addon.identifier, Addon.identifier, summary, symbols)

end