do

local Addon, private = ...

local catalog = { }

catalog["zlib.deflate"] = {
	summary = [[
Returns a "stream" function that compresses (or deflates) all strings passed in.]],
	signatures = {
		"stream = zlib.deflate(compression_level) -- stream <- number",
		"deflated, eof, bytes_in, bytes_out = stream(input) -- string, boolean, number, number <- string",
		"deflated, eof, bytes_in, bytes_out = stream(input, flags) -- string, boolean, number, number <- string, string",
	},
	parameter = {
		["compression_level"] = "If no compression_level is provided uses Z_DEFAULT_COMPRESSION (6), compression level is a number from 1-9 where zlib.BEST_SPEED is 1 and zlib.BEST_COMPRESSION is 9.",
		["input"] = "The data to be deflated (compressed).",
		["flags"] = [[
Optional flags to control the stream's behavior. Valid values are "sync", "full" or "finish".

A "sync" flush will force all pending output to be flushed to the return value and the output is aligned on a byte boundary, so that the decompressor can get all input data available so far.  Flushing may degrade compression for some compression algorithms and so it should be used only when necessary.

A "full" flush will flush all output as with "sync", and the compression state is reset so that decompression can restart from this point if previous compressed data has been damaged or if random access is desired. Using Z_FULL_FLUSH too often can seriously degrade the compression.

A "finish" flush will force all pending output to be processed and results in the stream become unusable.  Any future attempts to print anything other than the empty string will result in an error that begins with IllegalState.]],
	},
	result = {
		["stream"] = "A stream function used for inflating.",
		["eof"] = [[true if "finish" was specified, otherwise it is false.]],
		["bytes_in"] = "How many bytes of input have been passed to stream.",
		["bytes_out"] = "Number of bytes returned in deflated string chunks.",
	},
}

catalog["zlib.inflate"] = {
	summary = [[
Returns a "stream" function that decompresses (or inflates) all strings passed in.  Optionally specify a windowBits argument that is passed to inflateInit2(), see zlib.h for details about this argument.  By default, gzip header detection is done, and the max window size is used.

The resulting stream takes input and inflates and returns a portion of it.  If it detects the end of a deflation stream, then total will be the total number of bytes read from input and all future calls to stream() with a non empty string will result in an error that begins with IllegalState.

No flush options are provided since the maximal amount of input is always processed.]],
	signatures = {
		"stream = zlib.inflate() -- stream <- void",
		"stream = zlib.inflate(windowBits) -- stream <- number",
		"inflated, eof, bytes_in, bytes_out = stream(input) -- string, boolean, number, number <- string",
	},
	parameter = {
		["windowBits"] = "Optional argument passed to infalteInit2. See zlib.h for details.",
		["input"] = "The deflated data to be infalted (uncompressed).",
	},
	result = {
		["stream"] = "A stream function used for inflating compressed data.",
		["inflated"] = "The data resulting from inflating input.",
		["eof"] = [[true when the input string is determined to be at the "end of the file".]],
		["bytes_in"] = "How many bytes of input have been passed to stream.",
		["bytes_out"] = "Number of bytes returned in inflated string chunks",
	},
}

local summary = [[
Rift provides the Lua zlib implementation by Brimworks. This section contains the documentation for the subset of zlib provided in the Rift API.

Complete documentation can be found at <<#!copy:https://github.com/brimworks/lua-zlib>>.]]

_G[Addon.identifier].AddLibraryWithRiftLikeCatalogIndex("zlib", "zlib", summary, catalog)

end